@@ -7,6 +7,8 @@
 --
 -- extension processing module.
 --
+{-# LANGUAGE FlexibleContexts #-}
+
 module Data.X509.Ext
     ( Extension(..)
     -- * Common extension usually found in x509v3
@@ -24,6 +26,7 @@ module Data.X509.Ext
     , ReasonFlag(..)
     -- * Accessor turning extension into a specific one
     , extensionGet
+    , extensionGetE
     , extensionDecode
     , extensionEncode
     ) where
@@ -36,8 +39,8 @@ import Data.ASN1.BitArray
 import Data.List (find)
 import Data.X509.ExtensionRaw
 import Data.X509.DistinguishedName
+import Data.X509.Internal
 import Control.Applicative
-import Control.Monad.Error
 
 -- | key usage flag that is found in the key usage extension field.
 data ExtKeyUsageFlag =
@@ -77,6 +80,15 @@ extensionGet (Extensions (Just l)) = findExt l
                             Just (Right e) -> Just e
                             _              -> findExt xs
 
+-- | Get a specific extension from a lists of raw extensions
+extensionGetE :: Extension a => Extensions -> Maybe (Either String a)
+extensionGetE (Extensions Nothing)  = Nothing
+extensionGetE (Extensions (Just l)) = findExt l
+  where findExt []     = Nothing
+        findExt (x:xs) = case extensionDecode x of
+                            Just r         -> Just r
+                            _              -> findExt xs
+
 -- | Try to decode an ExtensionRaw.
 --
 -- If this function return:
@@ -181,6 +193,8 @@ data AltName =
     | AltNameDNS String
     | AltNameURI String
     | AltNameIP  B.ByteString
+    | AltNameXMPP String
+    | AltNameDNSSRV String
     deriving (Show,Eq,Ord)
 
 -- | Provide a way to supply alternate name that can be
@@ -236,26 +250,59 @@ instance Extension ExtCrlDistributionPoints where
     --extEncode (ExtCrlDistributionPoints )
 
 parseGeneralNames :: ParseASN1 [AltName]
-parseGeneralNames = do
-    c <- getNextContainer Sequence
-    r <- sequence $ map toStringy c
-    return r
+parseGeneralNames = onNextContainer Sequence $ getMany getAddr
   where
-        toStringy (Other Context 1 b) = return $ AltNameRFC822 $ BC.unpack b
-        toStringy (Other Context 2 b) = return $ AltNameDNS $ BC.unpack b
-        toStringy (Other Context 6 b) = return $ AltNameURI $ BC.unpack b
-        toStringy (Other Context 7 b) = return $ AltNameIP  b
-        toStringy b                   = throwError ("GeneralNames: not coping with anything else " ++ show b)
+        getAddr = do
+            m <- onNextContainerMaybe (Container Context 0) getComposedAddr
+            case m of
+                Nothing -> getSimpleAddr
+                Just r  -> return r
+        getComposedAddr = do
+            n <- getNext
+            case n of
+                OID [1,3,6,1,5,5,7,8,5] -> do -- xmpp addr
+                    c <- getNextContainerMaybe (Container Context 0)
+                    case c of
+                        Just [ASN1String cs] ->
+                            case asn1CharacterToString cs of
+                                Nothing -> throwParseError ("GeneralNames: invalid string for XMPP Addr")
+                                Just s  -> return $ AltNameXMPP s
+                        _ -> throwParseError ("GeneralNames: expecting string for XMPP Addr got: " ++ show c)
+                OID [1,3,6,1,5,5,7,8,7] -> do -- DNSSRV addr
+                    c <- getNextContainerMaybe (Container Context 0)
+                    case c of
+                        Just [ASN1String cs] ->
+                            case asn1CharacterToString cs of
+                                Nothing -> throwParseError ("GeneralNames: invalid string for DNSSrv Addr")
+                                Just s  -> return $ AltNameDNSSRV s
+                        _ -> throwParseError ("GeneralNames: expecting string for DNSSRV Addr got: " ++ show c)
+                OID unknown -> throwParseError ("GeneralNames: unknown OID " ++ show unknown)
+                _           -> throwParseError ("GeneralNames: expecting OID but got " ++ show n)
+
+        getSimpleAddr = do
+            n <- getNext
+            case n of
+                (Other Context 1 b) -> return $ AltNameRFC822 $ BC.unpack b
+                (Other Context 2 b) -> return $ AltNameDNS $ BC.unpack b
+                (Other Context 6 b) -> return $ AltNameURI $ BC.unpack b
+                (Other Context 7 b) -> return $ AltNameIP  b
+                _                   -> throwParseError ("GeneralNames: not coping with unknown stream " ++ show n)
 
 encodeGeneralNames :: [AltName] -> [ASN1]
 encodeGeneralNames names =
     [Start Sequence]
-    ++ map encodeAltName names
+    ++ concatMap encodeAltName names
     ++ [End Sequence]
-  where encodeAltName (AltNameRFC822 n) = Other Context 1 $ BC.pack n
-        encodeAltName (AltNameDNS n)    = Other Context 2 $ BC.pack n
-        encodeAltName (AltNameURI n)    = Other Context 6 $ BC.pack n
-        encodeAltName (AltNameIP n)     = Other Context 7 $ n
+  where encodeAltName (AltNameRFC822 n) = [Other Context 1 $ BC.pack n]
+        encodeAltName (AltNameDNS n)    = [Other Context 2 $ BC.pack n]
+        encodeAltName (AltNameURI n)    = [Other Context 6 $ BC.pack n]
+        encodeAltName (AltNameIP n)     = [Other Context 7 $ n]
+        encodeAltName (AltNameXMPP n)   = [Start (Container Context 0),OID[1,3,6,1,5,5,7,8,5]
+                                          ,Start (Container Context 0), ASN1String $ asn1CharacterString UTF8 n, End (Container Context 0)
+                                          ,End (Container Context 0)]
+        encodeAltName (AltNameDNSSRV n) = [Start (Container Context 0),OID[1,3,6,1,5,5,7,8,5]
+                                          ,Start (Container Context 0), ASN1String $ asn1CharacterString UTF8 n, End (Container Context 0)
+                                          ,End (Container Context 0)]
 
 bitsToFlags :: Enum a => BitArray -> [a]
 bitsToFlags bits = concat $ flip map [0..(bitArrayLength bits-1)] $ \i -> do
