@@ -13,7 +13,6 @@ module Data.X509.Internal
     -- * error handling
     , ErrT
     , runErrT
-    , throwError
     ) where
 
 import Data.ASN1.Types
@@ -26,6 +25,7 @@ runErrT = runExceptT
 type ErrT = ExceptT
 #else
 import Control.Monad.Error
+runErrT :: ErrorT e m a -> m (Either e a)
 runErrT = runErrorT
 type ErrT = ErrorT
 #endif
